
    #write the SMT-code with the definition of the logic, determining the assignments and all the needed
    #commands to result in the expected way -depending on the operation-
import sys
import Data
import re
import math

          
#class that generates the SMT code in the specified file
class SMT(object):
    def __init__(self, rawData, filename):
        self.data = Data.Data(rawData)
        self.operation = self.data.operation
        self.modal = self.data.modal
        self.nworlds = self.data.nworlds
        self.skeleton = self.data.skeleton
        if self.operation == "s":
            self.verbose = 1
        else :
            self.verbose = 0
        self.filename = filename
		
        self._buildPropLogic() #self.codePropLogic
        self._buildModalLogic() #self.codeModalLogic
        self._buildDeclarations() #self.codeDeclarations
        self._buildCodeFormulas() #self.codeFormulas
        self._buildShows() #self.shows
        self.commit()

		
    #writes the code to the specified file. Last function to be called
    def commit(self):
        FILE = open(self.filename,"w")
        if self.verbose == 1:
            FILE.write("""(set-option :produce-models true)
            """)
        #FILE.write("""(set-logic AUFLIRA)
        #""")
        FILE.write(self.codePropLogic)
        FILE.write(self.codeModalLogic)
        FILE.write(self.codeDeclarations)
        FILE.write("\n\n")
        FILE.write(self.codeFormulas)
        FILE.write("\n\n")
        FILE.write("""(check-sat)
        """)
        if self.verbose == 1:
            FILE.write(self.shows)
        FILE.close()
    
    
    
    def _buildPropLogic(self):
        comps = self.data.ordinalSum
        h = """
        ;; propositional level definitions
      
        ;propositional variable type definitions
        (define-sort pvar () (Array Int Real))
        (define-fun component ((x pvar)) Real
          (select x 1))
        (define-fun value ((x pvar)) Real
          (select x 2))
        (define-fun setComponent ((x pvar) (comp Real)) pvar
          (store x 1 comp))
        (define-fun setValue ((x pvar) (val Real)) pvar
          (store x 2 val))   
          ;truth const
        (declare-const pT pvar)
        (assert (and (= (component pT) (~ 1.0)) (= (value pT) 1.0)))
        (declare-const pF pvar)
        (assert (and (= (component pF) (~ 1.0)) (= (value pF) 0.0)))
        
        ;;RELATIONS
        ;eq(x,y) T if x = y, F else
        (define-fun eq ((x pvar) (y pvar)) Bool
            (and (= (component x) (component y)) (= (value x) (value y))))
            
            
        ;g (x,y) true if x > y, false otherwise
        (define-fun g ((x pvar) (y pvar)) Bool
            ;same component
            (ite (= (component x) (component y)) (< (value y) (value x))
            ;values in different components
            ;non-prod, prod
            (ite (and (= (component x) (~ 1.0)) (> (component y) (~ 1.0)))
            """
        for comp in comps:
            if comp[1] == "p":
                h = h + """ (ite (= (component y) """+str(comps.index(comp))+""".0) (<= """+ str(comp[0][1])+"""(value x))
                """
        h = h + """;this case in never reached 
        false """
        for comp in comps:
            if comp[1] == "p":
                h = h + """ )"""
        h = h + """
        ;prod, non-prod
        (ite (and (> (component x) (~ 1.0)) (= (component y) (~ 1.0)))
            """
        for comp in comps:
            if comp[1] == "p":
                h = h + """ (ite (= (component x) """+str(comps.index(comp))+""".0) (<=  (value y) """+ str(comp[0][0])+""")
                """
        h = h + """;this case in never reached 
        false """
        for comp in comps:
            if comp[1] == "p":
                h = h + """ )"""
        h = h + """
        ;prod-prod
        (< (component y) (component x))))))
       
        ;geq (x,y) true if x >= y, false otherwise
        (define-fun geq ((x pvar) (y pvar)) Bool
        	(or (eq x y) (g x y)))
            
        ;l (x,y) true if x < y, false otherwise
        (define-fun l ((x pvar) (y pvar)) Bool
        	(not (geq x y)))
    	
        ;leq (x, y) true if x <= y, false otherwise
        (define-fun leq ((x pvar) (y pvar)) Bool
        	(not (g x y)))
        
		
        ;;OPERATIONS
        ;pmin(x,y)
        (define-fun pmin ((x pvar) (y pvar)) pvar
        	(ite (leq x y) x y))

        ;pmax(x,y)
        (define-fun pmax ((x pvar) (y pvar)) pvar
        	(ite (geq x y) x y))
            
        ;weak conjunction = pmin (x, y)
        (define-fun pwcon ((x pvar) (y pvar)) pvar
            (pmin x y))

        ;weak disjunction pmax (x, y)
        (define-fun pwdis ((x pvar) (y pvar)) pvar
            (pmax x y))
        
        ;delta pdelta = 1 iff x = 1, 0 oth.
        (define-fun pdelta ((x pvar)) pvar
            (ite (= (value x) 1.0) pT pF))
        
        ;pcon(x, y)
        (define-fun pcon ((x pvar) (y pvar)) pvar
        """
        for comp in comps:
            if comp[1] == "l":
            #the non-product components are closed
                h = h + """(ite (and (<= """+str(comp[0][0])+""" (value x)) (<= """+str(comp[0][0])+""" (value y))
                                 (<= (value x) """+str(comp[0][1])+""") (<= (value y) """+str(comp[0][1])+"""))
                            ;they are in the same luk-component
                            (setValue x
                            (ite (<= """ + str(comp[0][0]) + """ (- (+ (value x) (value y)) """ + str(comp[0][1]) + """))
                            (- (+ (value x) (value y)) """ + str(comp[0][1]) + """) """+str(comp[0][0])+"""))
                                 """
            elif comp[1] == "g":
                h = h + """(ite (and (<= """+str(comp[0][0])+""" (value x)) (<= """+str(comp[0][0])+""" (value y))
                                 (<= (value x) """+str(comp[0][1])+""") (<= (value y) """+str(comp[0][1])+"""))
                                 ;they are in the same god-component
                                 (setValue x
                                 (ite (<= (value x) (value y)) (value x) (value y)))
                                 """
            else:
            #the product components. here, we just check if the component indexes of the pvars coincide
                h = h + """(ite (and (= (component x) """+str(comps.index(comp))+""".0) (= (component y) """+str(comps.index(comp))+""".0))
                                ;they are in the same prod-component
                                (setValue x (+ (value x) (value y)))
                                """
        #the case where the variables belong to different components
        h = h + """;different components
        (pmin x y)"""
        for comp in comps:
            h = h + """)"""
        h = h + """)
        """

        h = h + """
        ;pimpl(x y) -residuum-
        (define-fun pimpl ((x pvar) (y pvar)) pvar
            (ite (leq x y) pT
            """
        for comp in comps:
            if comp[1] == "l":
            #the non-product components are closed
                h = h + """(ite (and (<= """+str(comp[0][0])+""" (value x)) (<= """+str(comp[0][0])+""" (value y))
                                 (<= (value x) """+str(comp[0][1])+""") (<= (value y) """+str(comp[0][1])+"""))
                                 ;they are in the same luk-component
                                 (setValue x (+ """+ str(comp[0][1]) +""" (- (value y) (value x))))
                                 """
            elif comp[1] == "g":
                h = h + """(ite (and (<= """+str(comp[0][0])+""" (value x)) (<= """+str(comp[0][0])+""" (value y))
                                 (<= (value x) """+str(comp[0][1])+""") (<= (value y) """+str(comp[0][1])+"""))
                                 ;they are in the same god-component
                                 y
                                 """
            else:
            #the product components. here, we just check if the component indexes of the pvars coincide
                h = h + """(ite (and (= (component x) """+str(comps.index(comp))+""".0) (= (component y) """+str(comps.index(comp))+""".0))
                                ;they are in the same prod-component
                                (setValue x (- (value y) (value x)))
                                """
        #the case where the variables belong to different components
        h = h + """;different components
        y)"""
        for comp in comps:
            h = h + """)"""
        h = h + """)
        """


        h = h + """
        ;negation (pneg x = x -> 0)
        (define-fun pneg ((x pvar)) pvar
            (pimpl x pF)
        )"""
        
        self.codePropLogic = h


    def _buildModalLogic(self):
        comps = self.data.ordinalSum

        h = """
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;modal types and functions

        (define-sort var () (Array Int pvar))
        (define-fun valueInWorld ((x var) (world Int)) pvar
            (select x world)
        )
        
        ;truth constants
        (declare-const T var)
        """
        for i in range(self.nworlds):
            h = h + """(assert (= (valueInWorld T """+str(i)+""") pT))"""
        h = h + """
        (declare-const F var)"""
        for i in range(self.nworlds):
            h = h + """(assert (= (valueInWorld F """+str(i)+""") pF))"""
        
        h = h + """
        
        ;auxiliar var to generate the results
        (declare-const results var)	
              
        ;weak conjunction
        (define-fun wcon ((x var) (y var)) var
            """
        for i in range(self.nworlds):
            h = h + """(store """
        h = h + """results """ + str(self.nworlds-1) + """ (pwcon (select x """ + str(self.nworlds-1) + """) (select y """ + str(self.nworlds-1) + """)))"""
        for i in reversed(range(self.nworlds-1)):
            h = h + """ """ + str(i)+""" """+ """ (pwcon (select x """ + str(i) + """) (select y """ + str(i) + """)))"""
            
        h = h + """)
      
        ;weak disjunction
        (define-fun wdis ((x var) (y var)) var
            """
        for i in range(self.nworlds):
            h = h + """(store """
        h = h + """results """ + str(self.nworlds-1) + """ (pwdis (select x """ + str(self.nworlds-1) + """) (select y """ + str(self.nworlds-1) + """)))"""
        for i in reversed(range(self.nworlds-1)):
            h = h + """ """ + str(i)+""" """+ """ (pwdis (select x """ + str(i) + """) (select y """ + str(i) + """)))"""
        h = h + """)
         
        ;delta
        (define-fun delta ((x var)) var
            """
        for i in range(self.nworlds):
            h = h + """(store """
        h = h + """results """ + str(self.nworlds-1) + """ (pdelta (select x """ + str(self.nworlds-1) + """)))"""
        for i in reversed(range(self.nworlds-1)):
            h = h + """ """ + str(i)+""" """+ """ (pdelta (select x """ + str(i) + """)))"""

        h = h + """)
        
        ;con(x, y)
        (define-fun con ((x var) (y var)) var
            """
        for i in range(self.nworlds):
            h = h + """(store """
        h = h + """results """ + str(self.nworlds-1) + """ (pcon (select x """ + str(self.nworlds-1) + """) (select y """ + str(self.nworlds-1) + """)))"""
        for i in reversed(range(self.nworlds-1)):
            h = h + """ """ + str(i)+""" """+ """ (pcon (select x """ + str(i) + """) (select y """ + str(i) + """)))"""
            
        h = h + """)
        

        ;impl(x y) -residuum-
        (define-fun impl ((x var) (y var)) var
            """
        for i in range(self.nworlds):
            h = h + """(store """
        h = h + """results """ + str(self.nworlds-1) + """ (pimpl (select x """ + str(self.nworlds-1) + """) (select y """ + str(self.nworlds-1) + """)))"""
        for i in reversed(range(self.nworlds-1)):
            h = h + """ """ + str(i)+""" """+ """ (pimpl (select x """ + str(i) + """) (select y """ + str(i) + """)))"""
            
        h = h + """)
        
        ;negation (neg x = x -> 0)
        (define-fun neg ((x var)) var
            """
        for i in range(self.nworlds):
            h = h + """(store """
        h = h + """results """ + str(self.nworlds-1) + """ (pneg (select x """ + str(self.nworlds-1) + """)))"""
        for i in reversed(range(self.nworlds-1)):
            h = h + """ """ + str(i)+""" """+ """ (pneg (select x """ + str(i) + """)))"""
            
        h = h + """)
        

        ;MODAL OPERATIONS
        ;accessibility relation matrix
        ;the values will meet conditions from [skeleton]
        (declare-fun R () (Array Int (Array Int pvar)))
        ;some relations are always false from the skeleton construction. The others, are prop. variables, with the restrictions of the t-norm.
        """
        
        for i in range(self.nworlds):
            for j in range(self.nworlds):
                if self.skeleton[i][2][j] == 0:
                    h = h + """(assert (= pF (select (select R """+str(i)+""") """+ str(j)+""")))"""
                else:
                    #restriction of R as a prop.variable (see _buildDeclarations)
                    # (esto es para las variables modales) for w in range(self.nworlds):
                
                    someProd = [c for c in comps if c[1] == "p"] != []
                
                    if someProd:
                        h = h + """(assert (or
                            ;real values
                            (and (= (component (select (select R """+str(i)+""") """+ str(j)+""")) (~ 1.0))
                                    (or """
                        bot = 0.0
                        for comp in comps:
                            #real values: those between product components
                            if comp[1] == "p":
                                top = comp[0][0]
                                h = h + """(and (<= """+str(bot)+""" (value (select (select R """+str(i)+""") """+ str(j)+""")))
                                            (<= (value (select (select R """+str(i)+""") """+ str(j)+""")) """+str(top)+"""))
                                            """
                                bot = comp[0][1]
                        h = h + """(and (<= """+str(bot)+""" (value (select (select R """+str(i)+""") """+ str(j)+""")))
                                        (<= (value (select (select R """+str(i)+""") """+ str(j)+""")) 1.0))))
                                        """
                        h = h + """
                        ;non real values, i.e., those from the product components
                        (or """
                        for comp in comps:
                            if comp[1] == "p":
                                h = h + """(and (= (component (select (select R """+str(i)+""") """+ str(j)+""")) """+ str(comps.index(comp))+""".0)
                                            (< (value (select (select R """+str(i)+""") """+ str(j)+""")) 0.0))
                                            """
                        h = h + """)))"""
                    else:
                    #there are no product components -> all are reals, in [0,1]
                        h = h + """(assert (and (= (component (select (select R """+str(i)+""") """+ str(j)+""")) (~ 1.0))
                                            (<= 0.0 (value (select (select R """+str(i)+""") """+ str(j)+""")))
                                            (<= (value (select (select R """+str(i)+""") """+ str(j)+""")) 1.0)))"""
    
    
    
                    h = h + """
                    ;limitations for the finitely valued components
                    """
                    for c in comps:
                        if c[2] != 0:
                            #finite valued component
                            posValues = ["(+  "+ str(c[0][0])+ " (* "+str(c[0][1]-c[0][0])+" (/ "+str(float(i))+ " " +str(float(c[2]-1))+")))" for i in range(c[2])]
                            #determine the possible values in that part
                            h = h + """(assert (ite (and (<= """+str(c[0][0])+""" (value (select (select R """+str(i)+""") """+ str(j)+""")))
                                                        (>= """+str(c[0][1])+""" (value (select (select R """+str(i)+""") """+ str(j)+"""))))
                                                    (or """
                            for v in posValues:
                                h = h + """ (= (value (select (select R """+str(i)+""") """+ str(j)+""")) """+v+""")
                                """
                            h = h + """) true))
                            """
                    #h = h + """(assert (and (<= 0 (select (select R """+str(i)+""") """+ str(j)+""")) (<= (select (select R """+str(i)+""") """+ str(j)+""") 1.0)))"""
                  
        h = h + """
        ;; auxiliary functions
        
        ;box related functions
        
        ;given a var and an index i, returns a list of reals of length nworlds (i.e., a var) with 
        ;at each index p, the value R[i][p] -> var[p]
        (define-fun tfunc ((x var) (orig Int)) var
        """		
        # for i in range(self.nworlds):
        #     h = h + """(store """
        # h = h + """results """+ str(self.nworlds-1)+""" (ite (not (select (select R orig) """+str(self.nworlds-1)+""")) pT (select x """+str(self.nworlds-1)+""")))"""
        # for i in reversed(range(self.nworlds-1)):
        #     h = h + """ """+str(i)+""" (ite (not (select (select R orig) """+str(i)+""")) pT (select x """+str(i)+""")))"""


        for i in range(self.nworlds):
            h = h + """(store """
        h = h + """results """+ str(self.nworlds-1)+""" (pimpl (select (select R orig) """+str(self.nworlds-1)+""") (select x """+str(self.nworlds-1)+""")))"""
        for i in reversed(range(self.nworlds-1)):
            h = h + """ """+str(i)+""" (pimpl (select (select R orig) """+str(i)+""") (select x """+str(i)+""")))"""
        
        h = h + """)
        
        ;returns the minimum of a list of variables of lentgh nworlds  
        (define-fun minList ((x var)) pvar
        """
        #min = 1
        if self.nworlds == 1:
          h = h + """(select x 0))"""
        else:
          for j in range(self.nworlds-1):
              h = h + """(pmin """    
          h = h + """(select x 0) (select x 1))"""
          for j in range(self.nworlds-2):
              h = h + """ (select x """+str(j+2)+"""))"""
          h = h + """)""" 
          
        h = h + """
        
        ;minSuccessors
        ;takes a variable and an index and returns the corresponging pVar with the value of the minimum 
        ;over the successors of the world given by the index
        (define-fun minSuccessors ((x var) (i Int)) pvar
          (minList (tfunc x i)))
        
        ;box
        (define-fun box ((x var)) var 
        """
        for i in range(self.nworlds):
            h = h + """ (store"""
        h = h + """ results """+ str(self.nworlds-1) + """ (minSuccessors x """ + str(self.nworlds-1)+"""))"""
        for i in reversed(range(self.nworlds-1)):
            h = h + """ """+str(i)+""" (minSuccessors x """ + str(i) +"""))"""
        
        h = h + """)
        
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;diamond functions
        
        ;given a var and an index i, returns a list of reals of length nworlds (i.e., a var) with 
        ;at each index p, the value R[i][p] * var[p]
        (define-fun bfunc ((x var) (orig Int)) var
        """		
        for i in range(self.nworlds):
            h = h + """(store """
        h = h + """results """+ str(self.nworlds-1)+""" (pcon (select (select R orig) """+str(self.nworlds-1)+""") (select x """+str(self.nworlds-1)+""")))"""
        for i in reversed(range(self.nworlds-1)):
            h = h + """ """+str(i)+""" (pcon (select (select R orig) """+str(i)+""") (select x """+str(i)+""")))"""


        # for i in range(self.nworlds):
        #     h = h + """(store """
        # h = h + """ results """+ str(self.nworlds-1)+""" (ite (not (select (select R orig) """+str(self.nworlds-1)+""")) pF (select x """+str(self.nworlds-1)+""")))"""
        # for i in reversed(range(self.nworlds-1)):
        #   h = h + """ """+str(i)+""" (ite (not (select (select R orig) """+str(i)+""")) pF (select x """+str(i)+""")))"""

        h = h + """)
          
        ;returns the maximum of a list of reals of lentgh nworlds  
        (define-fun maxList ((x var)) pvar
        """
        #min = 1
        if self.nworlds == 1:
            h = h + """(select x 0))"""
        else:
            for j in range(self.nworlds-1):
                h = h + """(pmax """
            h = h + """(select x 0) (select x 1))"""
            for j in range(self.nworlds-2):
                h = h + """ (select x """+str(j+2)+"""))"""
            h = h + """)"""
          
        h = h + """
        
        ;maxSuccessors
        ;takes a variable and an index and returns the corresponging pVar with the value of the maximum 
        ;over the successors of the world given by the index
        (define-fun maxSuccessors ((x var) (i Int)) pvar
          (maxList (bfunc x i)))
        
        ;diamond
        (define-fun diamond ((x var)) var 
        """
        for i in range(self.nworlds):
            h = h + """ (store """
        h = h + """ results """+ str(self.nworlds-1) + """ (maxSuccessors x """ + str(self.nworlds-1)+"""))"""
        for i in reversed(range(self.nworlds-1)):
            h = h + """ """+str(i)+""" (maxSuccessors x """ + str(i) +"""))"""
        
        h = h + """)
        """

        self.codeModalLogic = h

    def _buildDeclarations(self):
        #assign the variables
        comps = self.data.ordinalSum
        h = """
        ;;DECLARATIONS
        ;;;assingments of the variables
        """
        for variab in self.data.variables:        
            h = h + """(declare-fun """ + variab + """ () var)
            """
            #(declare-fun """ + variab + """i () (Array Int (Array Int Int)))
            #"""
            for w in range(self.nworlds):
                
                someProd = [c for c in comps if c[1] == "p"] != []
                
                if someProd:
                    h = h + """(assert (or
                        ;real values
                        (and (= (component (valueInWorld """+variab+""" """+str(w)+""")) (~ 1.0))
                                (or """
                    bot = 0.0
                    for comp in comps:
                        #real values: those between product components
                        if comp[1] == "p":
                            top = comp[0][0]
                            h = h + """(and (<= """+str(bot)+""" (value (valueInWorld """+variab+""" """+str(w)+""")))
                                        (<= (value (valueInWorld """+variab+""" """+str(w)+""")) """+str(top)+"""))
                                        """
                            bot = comp[0][1]
                    h = h + """(and (<= """+str(bot)+""" (value (valueInWorld """+variab+""" """+str(w)+""")))
                                        (<= (value (valueInWorld """+variab+""" """+str(w)+""")) 1.0))))
                                        """
                    h = h + """
                    ;non real values, i.e., those from the product components
                    (or """
                    for comp in comps:
                        if comp[1] == "p":
                            h = h + """(and (= (component (valueInWorld """+variab+""" """+str(w)+""")) """+ str(comps.index(comp))+""".0)
                                        (< (value (valueInWorld """+variab+""" """+str(w)+""")) 0.0))
                                        """
                    h = h + """)))"""
                else:
                #there are no product components -> all are reals, in [0,1]
                    h = h + """(assert (and (= (component (valueInWorld """+variab+""" """+str(w)+""")) (~ 1.0))
                                        (<= 0.0 (value (valueInWorld """+variab+""" """+str(w)+""")))
                                        (<= (value (valueInWorld """+variab+""" """+str(w)+""")) 1.0)))"""
    
    
    
                h = h + """
                ;limitations for the finitely valued components
                """
                for c in comps:
                    if c[2] != 0:
                        #finite valued component
                        posValues = ["(+  "+ str(c[0][0])+ " (* "+str(c[0][1]-c[0][0])+" (/ "+str(float(i))+ " " +str(float(c[2]-1))+")))" for i in range(c[2])]
                        #determine the possible values in that part
                        h = h + """(assert (ite (and (<= """+str(c[0][0])+""" (value (valueInWorld """+ variab + """ """ + str(w)+""")))
                                                     (>= """+str(c[0][1])+""" (value (valueInWorld """+ variab + """ """ + str(w)+"""))))
                                                (or """
                        for v in posValues:
                            h = h + """ (= (value (valueInWorld """+variab+ """ """ + str(w)+""")) """+v+""")
                            """
                        h = h + """) true))
                        """
                                                

        h = h + """
        
        ;declaration of the constants
        """
        #print "constantsInProducts:"
        #print self.data.constantsInProducts
        
        #print "constantsMap:"
        #print self.data.constantsMap
        
        
        
        #assign the constants
        #constants within any product component
        constInProds = []
        for indexPList in self.data.constantsInProducts:
            for c in self.data.constantsInProducts[indexPList]:
                constInProds.append(c)
        
        namesCs = [x[0] for x in constInProds]
        
        for c in [x for x in self.data.constantsMap if x not in namesCs]:
            #format of the constants [name, value in [0,1]]
            h = h + """
            ;constant outside any product component
            ;WARNING: no control of constants in the finitely-valued components is done!
            
            (declare-const """+ c+""" var )
            (assert (and (= (value (valueInWorld """ +c+""" 0)) """+ str(self.data.constantsMap[c]) + """)
                (= (component (valueInWorld """ + c+""" 0)) (~ 1.0))))
            """
            for k in range(self.nworlds)[1:]:
                h = h + """(assert (= (valueInWorld """ +c+""" """+str(k)+""") (valueInWorld """ +c+""" 0)))
                """
        for indexPList in self.data.constantsInProducts:
            flag = 0
            for c in self.data.constantsInProducts[indexPList]:
                #print "constantInP:"
                #print c
                h = h + """
                ;constant in product component """+str(indexPList)+"""
                
                (declare-fun """+ c[0]+"""() var )
                (assert (= (component (valueInWorld """+c[0]+""" 0)) """+ str(indexPList) + """.0))
                    """
#                for k in range(self.nworlds):
#                    h = h + """(assert (= (component (valueInWorld """+c[0]+""" """+str(k)+""")) """+ str(indexPList) + """.0))
#                    """
                if self.data.constantsInProducts[indexPList].index(c) == 0:
                    #the first constant of the component is taken as reference for the others
                    ref = c
                    #the value in world 0 is free in R-, the rest of the array coincides with it -that is the meaning of a constant in modal-
                    h = h + """
                    ;it is the reference constant of the component
                    (assert (< (value (valueInWorld """+c[0]+""" 0)) 0.0))
                    """
                            
                    for k in range(self.nworlds-1):
                        h = h + """(assert (= (valueInWorld """+c[0]+""" """+str(k+1)+""") (valueInWorld """+c[0]+""" 0)))
                        """
                else :
						  if flag == 0:
								print """WARNING: you have specified more than one constant element in product component """+str(indexPList)+""".
Because of the impossibility of dealing with non-algebraic numbers, the results may be approximated."""
								flag = 1
						  else:
								log = math.log((c[1] - self.data.ordinalSum[indexPList][0][0])/(self.data.ordinalSum[indexPList][0][1] - self.data.ordinalSum[indexPList][0][0]))/math.log((ref[1] - self.data.ordinalSum[indexPList][0][0])/(self.data.ordinalSum[indexPList][0][1] - self.data.ordinalSum[indexPList][0][0]))
                    
								h = h + """
                    ;constant relativized to the reference one
                    (assert (= (value (valueInWorld """+c[0]+""" 0)) (* (value (valueInWorld """+ ref[0]+""" 0)) """+ str(log)+ """)))
                    """

								for k in range(self.nworlds-1):
									 h = h + """(assert (= (valueInWorld """+c[0]+""" """+str(k+1)+""") (valueInWorld """+c[0]+""" 0)))
                        """

        h = h + """
        
        ;the values of parts of the functions are fixed following the skeleton definition
        """
        
        if self.nworlds > 1:
            for i in range(self.nworlds-1):
                j = 1
                modType = self.skeleton[i+1][0][j]

                while self.skeleton[i+1][0][j] != " ":
                    j = j +1
                nonModalPart = self.skeleton[i+1][0][j+1:-1]
                if modType == "b":
                    h = h + """
                        (assert (= (select """+self.skeleton[i+1][0]+""" """+str(self.skeleton[i+1][1])+""")
                            (pimpl (select (select R """+str(self.skeleton[i+1][1])+""") """+str(i+1)+""")
                                (select """+nonModalPart+""" """+str(i+1)+"""))))"""
                else:
                    #if type is "d", i.e., diamond operation
                    h = h + """
                        (assert (= (select """+self.skeleton[i+1][0]+""" """+str(self.skeleton[i+1][1])+""")
                            (pcon (select (select R """+str(self.skeleton[i+1][1])+""") """+str(i+1)+""")
                                (select """+nonModalPart+""" """+str(i+1)+"""))))"""

                         
        self.codeDeclarations = h
        
        
    def _buildCodeFormulas(self):
        h = """;;;PROBLEM
        """
        if self.operation == "th":
            #to check a theorem, assert its value is less than 1 and and check this is unsat
            h = h + """(assert (< (value (valueInWorld """ + self.data.dataOperation[0]+""" 0)) 1.0))"""
        elif self.operation == "d":
            #similary, assert 1 to the premises and <1 to the conclussion and check this is unsat
            for i in range(len(self.data.dataOperation)):
                if i < len(self.data.dataOperation)-1:
                    h = h + """(assert (= (value (valueInWorld """ + self.data.dataOperation[i]+""" 0)) 1.0))
                    """
                else :
                    h = h + """(assert (< (value (valueInWorld """ + self.data.dataOperation[i]+""" 0)) 1.0))
                    """
        elif self.operation == "s":
            for eq in self.data.spEquations:
                #print "equations: "
                #print eq
                h = h + """(assert ("""+eq[0]+""" (valueInWorld """+eq[1][0]+""" 0) (valueInWorld """+eq[1][1]+""" 0)))
                """
     
        self.codeFormulas = h
     
         
    def _buildShows(self):
    	h = """;SHOWS
        """
        #first, we read the value assigned to the first constant in list for each product component - in case it has any
        for j in self.data.constantsInProducts:
            c = self.data.constantsInProducts[j][0]
            h = h + """(get-value((value (select """+c[0]+""" 0))))
                    """

        #values of R
    	for i in range(self.nworlds):
            for j in range(self.nworlds):
                h = h + """(get-value((component (select (select R """+str(i)+""") """+str(j)+"""))))
                    """
                h = h + """(get-value((value (select (select R """+str(i)+""") """+str(j)+"""))))
                    """

#        
#        for j in range(len(self.data.ordinalSum)):
#            if self.data.ordinalSum[j][1] == "p":
#                if self.data.constantsIn[j] != []:
#                    c = self.data.constantsIn[j][0]
#                    h = h + """(get-value((value (select """+c[0]+""" """+str(j)+"""))))
#                    """
        #then we read all the values (in each world) of each variable
        for v in self.data.variables:
	     for i in range(self.nworlds):
                h = h + """(get-value((component (select """+v+""" """+str(i)+"""))))
                """
                h = h + """(get-value((value (select """+v+""" """+str(i)+"""))))
                """

        self.shows = h

   